using System;
using System.Collections.Generic;
using NUnit.Framework;
using Rhino.Mocks;
using Rhino.Mocks.Constraints;
using Rhino.Mocks.Interfaces;
using RhinoMockDemo;
using RhinoMockDemo.DBOs;
using RhinoMockDemo.Providers;

namespace TestLib
{
	[TestFixture]
	public class CustomerPresenterTests
	{
		private MockRepository _mock;
		private CustomerPresenter _presenter;
		private ICustomerDbo _dboMock;
		private ICustomerView _view;
		private IClassFactory _factory;


		private void SetupMockObjectsAndSystemUnderTest()
		{
			_mock = new MockRepository();
			_dboMock = _mock.DynamicMock<ICustomerDbo>();
			_view = _mock.DynamicMock<ICustomerView>();
			_factory = _mock.DynamicMock<IClassFactory>();
			_presenter = new CustomerPresenter(_view, _factory, _dboMock);
		}

		[Test]
		public void How_to_test_a_method_passing_data_from_one_mock_to_the_next()
		{
			SetupMockObjectsAndSystemUnderTest();
			
			// create data objects
			List<ICustomer> oList = new List<ICustomer>();
			
			// tell the mock objects what to expect and return when called
			using(_mock.Record())
			{
				Expect.Call(_dboMock.GetAllCustomers()).Return(oList);
				
				//both lines below do the same thing.
				// _view.Customers = oList;   // old way to set a property expectation on a mocked object
				Expect.Call(delegate { _view.Customers = oList; });  // new way to set a property expectation
			}
			using(_mock.Playback())
			{
				_presenter.Initialize();
			}
		}


		[Test]
		public void How_to_test_a_method_call()
		{
			SetupMockObjectsAndSystemUnderTest();
			const int CUSTOMER_ID = 1;
			
			using(_mock.Record())
			{
				_dboMock.DeleteCustomer(CUSTOMER_ID);
			}
			using(_mock.Playback())
			{
				_presenter.DeleteCustomer(CUSTOMER_ID);
			}
		}

		[Test]
		public void How_to_test_a_function_return()
		{
			SetupMockObjectsAndSystemUnderTest();
			ICustomer c = _mock.CreateMock<ICustomer>();
			using (_mock.Record())
			{
				Expect.Call(_dboMock.CreateCustomer()).Return(c);
			}
			using (_mock.Playback())
			{
				Assert.IsNotNull(_presenter.CreateCustomer());
			}
		}

		[Test]
		public void Full_Name_with_prefix_Test()
		{
			SetupMockObjectsAndSystemUnderTest();
		
			ICustomer c = _mock.DynamicMock<ICustomer>();

			using (_mock.Record())
			{
				Expect.Call(c.Title).Return("Mr.").Repeat.Any();
				Expect.Call(c.FirstName).Return("Chris").Repeat.Any();
				Expect.Call(c.LastName).Return("Brandsma").Repeat.Any();

			}
			using (_mock.Playback())
			{
				Assert.AreEqual("Mr. Chris Brandsma", _presenter.FullName(c));
			}
		}
		
		[Test]
		public void Full_Name_with_out_prefix_Test()
		{
			SetupMockObjectsAndSystemUnderTest();
		
			ICustomer c = _mock.DynamicMock<ICustomer>();

			using (_mock.Record())
			{
				Expect.Call(c.Title).Return("").Repeat.Any();
				Expect.Call(c.FirstName).Return("Chris").Repeat.Any();
				Expect.Call(c.LastName).Return("Brandsma").Repeat.Any();
			}
			using (_mock.Playback())
			{
				Assert.AreEqual("Chris Brandsma", _presenter.FullName(c));
			}
		}

		[Test]
		public void How_To_Handle_An_Event_From_A_Class_Test()
		{
			SetupMockObjectsAndSystemUnderTest();
			using(_mock.Record())
			{
				_dboMock.MyEvent += null;
				LastCall.IgnoreArguments();
			}
			using(_mock.Playback())
			{
				_presenter.HandlesEvent();
			}
		}
		
		private IEventRaiser _eventRaiser;
		[Test]
		public void How_To_Fire_An_Event_From_A_Mocked_Class()
		{
			SetupMockObjectsAndSystemUnderTest();
			
			using(_mock.Record())
			{
				_dboMock.MyEvent += null;
				LastCall.Constraints(Is.NotNull());
				_eventRaiser = LastCall.GetEventRaiser();
				// Initialize method expectations
				List<ICustomer> oList = new List<ICustomer>();
				using (_mock.Ordered())
				{
					Expect.Call(_dboMock.GetAllCustomers()).Return(oList);
					_view.Customers = oList;
				}
			}
			using(_mock.Playback())
			{
				_presenter.HandlesEvent();
				_eventRaiser.Raise(null, null);
			}
		}
	}
}