using System;
using System.Collections.Generic;
using NUnit.Framework;
using Rhino.Mocks;
using Rhino.Mocks.Interfaces;
using RhinoMockDemo;
using RhinoMockDemo.DBOs;
using RhinoMockDemo.Providers;

namespace TestLib
{
	[TestFixture]
	public class CustomerPresenterTests_backup2
	{
		private MockRepository _mock;
		private CustomerPresenter _presenter;
		private ICustomerDbo _dbo;
		private ICustomerView _view;
		private IClassFactory _factory;

		[SetUp]
		public void Setup()
		{
			_mock = new MockRepository();
			_factory = _mock.CreateMock<IClassFactory>();
			_dbo = _mock.CreateMock<ICustomerDbo>();
			_view = _mock.CreateMock<ICustomerView>();
			_presenter = new CustomerPresenter(_view, _factory, _dbo);
		}

		[TearDown]
		public void TearDown()
		{
			_mock.VerifyAll();
		}

		[Test]
		public void InitializeTest()
		{
			List<ICustomer> oList = new List<ICustomer>();
			Expect.Call(_dbo.GetAllCustomers()).Return(oList).Repeat.AtLeastOnce();
			_view.Customers = oList;
			//
			// set mocking expectations 
			//
			_mock.ReplayAll();
			//
			// run tests
			//
			_presenter.Initialize();
		}

		[Test]
		public void FullNameTest()
		{
			ICustomer c = _mock.CreateMock<ICustomer>();
			Expect.Call(c.FirstName).Return("Chris");
			Expect.Call(c.LastName).Return("Brandsma");
			Expect.Call(c.Title).Return("Mr.");
			//
			ICustomer c2 = _mock.CreateMock<ICustomer>();
			Expect.Call(c2.FirstName).Return("Chris");
			Expect.Call(c2.LastName).Return("Brandsma");
			Expect.Call(c2.Title).Return(""); //
			// set mocking expectations 
			//
			_mock.ReplayAll();
			//
			// run tests
			//
			Assert.AreEqual("Mr. Chris Brandsma", _presenter.FullName(c));
			Assert.AreEqual("Chris Brandsma", _presenter.FullName(c2));
		}

		[Test]
		public void DeleteCustomerTest()
		{
			_dbo.DeleteCustomer(1);
			//
			// set mocking expectations 
			//
			_mock.ReplayAll();
			//
			// run tests
			//
			_presenter.DeleteCustomer(1);
		}

		[Test]
		public void CreateCustomerTest()
		{
			ICustomer c = _mock.CreateMock<ICustomer>();
			Expect.Call(_dbo.CreateCustomer()).Return(c);
			//
			// set mocking expectations 
			//
			_mock.ReplayAll();
			//
			// run tests
			//
			ICustomer returnedCustomer = _presenter.CreateCustomer();
			Assert.IsNotNull(returnedCustomer);
		}

		[Test]
		public void EventHandlerTest()
		{
			_dbo.MyEvent += null;
			LastCall.IgnoreArguments();
			//
			// set mocking expectations 
			//
			_mock.ReplayAll();
			//
			// run tests
			//
			_presenter.HandlesEvent();
		}

		private IEventRaiser _eventRaiser;

		[Test]
		public void FiredEventTest()
		{
			_dbo.MyEvent += null;
			LastCall.Constraints(Is.NotNull());
			_eventRaiser = LastCall.GetEventRaiser();
			// handle Initialize method
			List<ICustomer> oList = new List<ICustomer>();
			Expect.Call(_dbo.GetAllCustomers()).Return(oList).Repeat.AtLeastOnce();
			_view.Customers = oList;
			//
			// set mocking expectations 
			//
			_mock.ReplayAll();
			//
			// run tests
			//
			_presenter.HandlesEvent();
			_eventRaiser.Raise(null, null);
		}
		
	}
}